/***************************************************************************
                          messagedispatcher.cpp  -  description
                             -------------------
    begin                : Fri Jan 19 2001
    copyright            : (C) 2001 by Matthias Welwarsky
    email                : matze@stud.fbi.fh-darmstadt.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kdebug.h>

#include "messagedispatcher.h"
#include "phoneconnection.h"
#include "phonecontrol.h"

MessageDispatcher::MessageDispatcher(PhoneConnection* pC){
	phoneConnection = pC;
	connect(pC, SIGNAL(responseReady(QString)), this, SLOT(receiveResponse(QString)));
	currentControl = NULL;
}

MessageDispatcher::~MessageDispatcher(){
}

/** called by the PhoneConnection when a complete response line is received from the mobile phone. */
void MessageDispatcher::receiveResponse(QString response){

	if (currentControl)
		emit responseReady(response);
}

/** called by the PhoneControl object to send a command to the mobile phone */
void MessageDispatcher::sendCommand(QString command, QObject* pC){
	if (currentControl && currentControl == pC)
		phoneConnection->sendCommand(command+QChar('\r'));
}

/** transmit a PDU to the mobile phone. the only difference to sendCommand is the line termination,
which is "CR" for normal commands, and "ESC" for PDUs */
void MessageDispatcher::sendPDU(QString pdu, QObject* pC){
	if (currentControl && currentControl == pC)
		phoneConnection->sendCommand(pdu+QChar(0x1a));
}

/** Acquires the dispatcher for a phone control. This must be done before sending messages, for locking
purposes. Arguments are a QObject* and a slot definition. All responses from the conversation with
the phone are emitted to the QObjects slot, until releaseDispatcher is called.  */
bool MessageDispatcher::acquireDispatcher(QObject* control, const char* slot){
	
	// check the lock
	if (currentControl != NULL)
		return false;
		
	currentControl = control;
	
	connect(this, SIGNAL(responseReady(QString)), control, slot);
	
	kdDebug() << "control acquired dispatcher\n";

	return true;
}

/** releases the dispatcher. The Phone Control can no longer send any messages to the phone unless it
acquires the dispatcher again. */
bool MessageDispatcher::releaseDispatcher(QObject* control){
	if (control != currentControl)
		return false;
		
	currentControl = NULL;
	disconnect(control);
	
	kdDebug() << "control released dispatcher\n";
	return true;
}
