/***************************************************************************
                          phonebookitem.cpp  -  description
                             -------------------
    begin                : Tue Jan 23 2001
    copyright            : (C) 2001 by Matthias Welwarsky
    email                : matze@stud.fbi.fh-darmstadt.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <kiconloader.h>

#include "phonebookitem.h"

PhonebookItem::PhonebookItem(QListView* parent)
	: QListViewItem(parent, "", "", "") {
	
	KIconLoader iconLoader;
	
	trashPixmap = iconLoader.loadIcon("edittrash", KIcon::Small);
	dirtyPixmap = iconLoader.loadIcon("remote", KIcon::Small);
	
	itemState = CLEAN;
}	

PhonebookItem::PhonebookItem(QListView *parent, QString name, QString number, int index, int type)
	: QListViewItem(parent, "", name, number) {

	theName = name;
	theNumber = number;
	theIndex = index;
	theType = type;
		
	KIconLoader iconLoader;
	
	trashPixmap = iconLoader.loadIcon("edittrash", KIcon::Small);
	dirtyPixmap = iconLoader.loadIcon("remote", KIcon::Small);
	
	itemState = CLEAN;
}

PhonebookItem::~PhonebookItem(){
}

/** set the name of the phonebook entry */
void PhonebookItem::setName(QString name){
	theName = name;
	setText(1, name);
	setState(CHANGED);	
}

/** set the number of the phonebook entry */
void PhonebookItem::setNumber(QString number) {
	theNumber = number;
	setText(2, number);
	setState(CHANGED);
}

/** set the index of the phonebook entry */
void PhonebookItem::setIndex(int index) {
	theIndex = index;
	setState(CHANGED);
}

/** returns true/false, according to whether the entry was edited. */
bool PhonebookItem::changed() const{
	return (state() != CLEAN);
}

/** returns a QString suitable as phonebook entry text, i.e. with escaped GSM
  * characters */
QString PhonebookItem::gsmEscape(const QString& text){
	QString result;
	int i;

	for (i = 0; i < text.length(); i++) {
		switch (text[i]) {
		default:
			result += text[i];
			break;
		case '"':
			result += "\\22";
			break;
		case 0x08:
			result += "\\08";
			break;
		case '@':
			result += "\\00";
			break;
		case '':
			result += '~';
			break;
		case '':
			result += '|';
			break;
		case '':
			result += '[';
			break;
		case '':
			result += '{';
			break;
		case '':
			result += "\\5c";
			break;
		case '':
			result += '^';
			break;
		}
	}
	return result;
}

/** returns a QString representing the argument, with removed GSM special characters */
QString PhonebookItem::gsmUnescape(const QString& text){
	QString result;
	int i;

	for (int i = 0; i < text.length(); i++) {
		switch (text[i]) {
		default:
			result += text[i];
			break;
		case '~':
			result += '';
			break;
		case '|':
			result += '';
			break;
		case '[':
			result += '';
			break;
		case '{':
			result += '';
			break;
		case '\\':
			result += '';
			break;
		case '^':
			result += '';
			break;
		}
	}
	return result;	
}
/** returns the type of the entry */
int PhonebookItem::type() const{
	return theType;
}

PhonebookItem::ItemState PhonebookItem::state() const {
	return itemState;
}

void PhonebookItem::setState(ItemState state) {
	itemState = state;
	
	switch (state) {
	case CLEAN:
		setPixmap(0, QPixmap());
		break;
		
	case CHANGED:
		setPixmap(0, dirtyPixmap);
		break;
		
	case DELETED:
		setPixmap(0, trashPixmap);
		break;
	};
}