/***************************************************************************
                          phonestatuscontrol.h  -  description
                             -------------------
    begin                : Wed Jan 17 2001
    copyright            : (C) 2001 by Matthias Welwarsky
    email                : matze@stud.fbi.fh-darmstadt.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef PHONESTATUSCONTROL_H
#define PHONESTATUSCONTROL_H

#include <qwidget.h>
#include "phonestatuscontroldlg.h"

class MessageDispatcher;
class QTimerEvent;

/** phone status display control. shows parameters of the phone like battery level,
 *  rssi a.s.o
 *  @author Matthias Welwarsky
 */

class PhoneStatusControl : public PhoneStatusControlDlg {
   Q_OBJECT
public:
	enum State { Idle=0, MessageSent, ExpectIMSI, ExpectManufacturer, ExpectSoftVersion };

	PhoneStatusControl(QWidget *parent=0, const char *name=0);
	virtual ~PhoneStatusControl();

 	/** stops the status control, reimplemented from PhoneControl */
  	virtual void stopControl();
  	
	/** starts the status control, reimplemented from PhoneControl */
  	virtual void startControl();

public slots:  	
  	/** reimplemented from PhoneControl */
  	virtual void deliverMessage(QString);

private: // Private attributes
  	/** holds a pointer to the message dispatcher */
  	MessageDispatcher* messageDispatcher;
  	/** timer divisor */
  	int timerCount;
	/** state of the control (Idle/MessageSent) */
	State controlState;

protected: // Protected methods
  	/** timer callback for regular status updates. called every 10 seconds if the
	 *  control is active */
  	virtual void timerEvent(QTimerEvent*);
private: // Private methods
  /** starts an update sequence to gather information from the mobile phone */
  void startUpdateSequence();
};

#endif
