/***************************************************************************
                          phonetransferlogo.h  -  description
                             -------------------
    begin                : Thu Jan 18 2001
    copyright            : (C) 2001 by Matthias Welwarsky
    email                : matze@stud.fbi.fh-darmstadt.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef PHONETRANSFERLOGO_H
#define PHONETRANSFERLOGO_H

#include <qwidget.h>

#include "phonetransferlogodlg.h"
#include "tpdu.h"

/**transfer a logo from or to the mobile phone
  *@author Matthias Welwarsky
  */

class FileBrowser;
class MessageDispatcher;
class QProgressDialog;
class KDirOperator;

class PhoneTransferLogo : public PhoneTransferLogoDlg  {
   Q_OBJECT
public: 
	PhoneTransferLogo(QWidget *parent=0, const char *name=0);
	virtual ~PhoneTransferLogo();

  	/** stops the transfer control, reimplemented from PhoneControl */
  	virtual void stopControl();

  	/** starts the transfer control, reimplemented from PhoneControl */
  	virtual void startControl();

public slots: // Public slots
  	/** reimplemented from PhoneControl */
  	virtual void deliverMessage(QString);
  	/** uploads the logo with the filename given by the argument */
  	void setUploadLogo(const QString&);
  	/** loads the logo currently stored in the preview into the mobile phone */
  	void uploadLogo();

private: // Private attributes
	/** possible states of the control */
	enum State { Idle=0, DownloadLogo, UploadLogo };
	
    /** a part to display the directory contents */
    FileBrowser* fileBrowser;

  	/** the message dispatcher to communicate with the mobile phone */
  	MessageDispatcher* messageDispatcher;
  	
  	/** the state of the control */
  	State controlState;
  	
  	/**  */
  	int expectNR;
  	
  	/**  */
  	int maxNR;

  	/** a dialog to display the progress of up- and downloads */
  	QProgressDialog* progressDialog;
  	
  	/** holds the data of the logo recently downloaded from the mobile phone. */
  	QByteArray currentLogoData;
  	
  	/** holds the data to be uploaded to the mobile phone. */
  	QByteArray previewLogoData;
  	
  	/** holds the logo in a list of TPDUs */
  	TPDU::List logoData;
  	
  	/**  */
  	QListIterator<TPDU> logoDataIterator;
  	
  	/**  */
  	int bmpSubtype;

private: // Private methods
  	/** download the currently used logo from the mobile phone */
  	void downloadLogo();
};

#endif
