/***************************************************************************
                          tpdu.h  -  description
                             -------------------
    begin                : Wed Feb 14 2001
    copyright            : (C) 2001 by Matthias Welwarsky
    email                : matze@stud.fbi.fh-darmstadt.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef TPDU_H
#define TPDU_H

#include <qcstring.h>
#include <qbitmap.h>
#include <qlist.h>

/**a class that holds the data of a PDU from or to the phone.
  *@author Matthias Welwarsky
  */

class TPDU {
public:
	/** a list of TPDUs */
	typedef QList<TPDU> List;
	
	/* constructs an empty TPDU */
	TPDU();
  	
  	/** constructs a TPDU from a QString containing a PDU
  	  * in hexadecimal representation
  	  */
   	TPDU(const QString&);
   	
	~TPDU();
	
  	/** assigns a QString containing a PDU in hexadecimal
  	  * representations to the TPDU object
  	  */
  	TPDU& operator=(const QString&);

  	/** converts the PDU from it's internal format into a QString
      * containing a PDU in hexadecimal representation.
      */
  	QString toQString() const;

  	/** converts a list of TPDUs to a QByteArray. */
  	static QByteArray toByteArray(const TPDU::List&);

  	/** returns the size of the stored PDU */
  	int size() const;

  	/** No descriptions */
  	char& operator[](int) const;
  /** constructs a TPDU from a QByteArray */
   TPDU(const QByteArray&);
  	
  	/** splits a QByteArray into a list of TPDUs with a given maximum size. */
  	static TPDU::List split(const QByteArray&, int);

private: // Private attributes
  	/** the internal data store */
  	QByteArray pduData;

  	/** holds the digits of the hexadecimal numbering system */
  	static const QString hexDigit;
};

#endif
